"use strict";

var _template = _interopRequireDefault(require("@babel/template"));

var t = _interopRequireWildcard(require("@babel/types"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildPromiseFromAwait = (0, _template.default)("\n  CALLER(ARGS).then(RESPONSE => {\n    return [null, RESPONSE];\n  })\n  .catch(ERROR => {\n    return [ERROR];\n  });\n");

module.exports = function () {
  function awaitToPromiseMethod(caller, args) {
    var ast = buildPromiseFromAwait({
      CALLER: caller,
      RESPONSE: t.identifier('resp'),
      ARGS: args,
      ERROR: t.identifier('error')
    });
    var exp = ast;
    return exp;
  }

  function updatePath(path, left, right) {
    if (!t.isArrayPattern(left) || !t.isAwaitExpression(right)) return;
    var rightNodeOperation = right.argument;

    if (t.isCallExpression(rightNodeOperation)) {
      var callee = rightNodeOperation.callee;
      var args = rightNodeOperation.arguments;

      if (t.isMemberExpression(callee)) {
        var object = callee.object,
            property = callee.property;
        var caller = "await ".concat(object.name, ".").concat(property.name);
        path.replaceWith(awaitToPromiseMethod(caller, args));
      } else {
        var _caller = "await ".concat(callee.name);

        path.replaceWith(awaitToPromiseMethod(_caller, args));
      }
    } else if (t.isIdentifier(rightNodeOperation)) {
      var _caller2 = "await ".concat(rightNodeOperation.name);

      path.replaceWith(awaitToPromiseMethod(_caller2, []));
    } else if (t.isNewExpression(rightNodeOperation)) {
      var _args = rightNodeOperation.arguments;

      var _caller3 = "await new ".concat(rightNodeOperation.callee.name);

      path.replaceWith(awaitToPromiseMethod(_caller3, _args));
    }
  }

  return {
    name: 'babel-plugin-better-async-await',
    visitor: {
      AwaitExpression: function AwaitExpression(path) {
        var left, right;

        if (t.isAssignmentExpression(path.parent)) {
          left = path.parent.left;
          right = path.parent.right;
        } else if (t.isVariableDeclarator(path.parent)) {
          left = path.parent.id;
          right = path.parent.init;
        }

        updatePath(path, left, right);
      }
    }
  };
};