"use strict";

var _babelPluginTester = _interopRequireDefault(require("babel-plugin-tester"));

var _index = _interopRequireDefault(require("./index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _babelPluginTester.default)({
  plugin: _index.default,
  tests: [{
    title: 'promise with identifier',
    code: "\n        async function test() {\n          const [err, resp] = await promise;\n        }\n      ",
    output: "\n        async function test() {\n          const [err, resp] = await promise().then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }, {
    title: 'promise with member expression and args',
    code: "\n        async function test() {\n          const [err, resp] = await api.getData(4);\n        }\n      ",
    output: "\n        async function test() {\n          const [err, resp] = await api.getData(4).then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }, {
    title: 'promise with a call expression',
    code: "\n        async function test() {\n          const [err, resp] = await getData(4);\n        }\n      ",
    output: "\n        async function test() {\n          const [err, resp] = await getData(4).then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }, {
    title: 'promise with a new expression',
    code: "\n        async function test() {\n          const [err, resp] = await new Promise((resolve, reject) => {\n            resolve(true);\n          });\n        }\n      ",
    output: "\n        async function test() {\n          const [err, resp] = await new Promise((resolve, reject) => {\n            resolve(true);\n          }).then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }, {
    title: 'promise with a just a promise and resolve',
    code: "\n        async function test() {\n          const [err, resp] = await Promise.resolve(true);\n        }\n      ",
    output: "\n        async function test() {\n          const [err, resp] = await Promise.resolve(true).then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }, {
    title: 'promise with left side not an array expression',
    code: "\n        async function test() {\n          const resp = await api.getData();\n        }\n      ",
    output: "\n        async function test() {\n          const resp = await api.getData();\n        }\n      "
  }, {
    title: 'promise with left side an assignment expression',
    code: "\n        async function test() {\n          let err, resp;\n          [err, resp] = await api.getData();\n        }\n      ",
    output: "\n        async function test() {\n          let err, resp;\n          [err, resp] = await api.getData().then(resp => {\n            return [null, resp];\n          }).catch(error => {\n            return [error];\n          });\n        }\n      "
  }]
});